@@ -43,7 +43,7 @@ public:
   QString defaultValue() const override;
   void setValue(const QString & value) override;
   void reset() override;
-  bool initFromText(const char * text, int & textLength) override;
+  bool initFromText(const QString & filterName, const char * text, int & textLength) override;
 
 private:
   QString _name;
