@@ -75,10 +75,12 @@ PreviewWidget::PreviewWidget(QWidget * parent) : QWidget(parent)
   _xPreviewSplit = 0.5f;
   _yPreviewSplit = 0.5f;
   _draggingMode = DraggingMode::Inactive;
+  _savedPreviewType = static_cast<PreviewType>(QSettings().value(PREVIEW_SPLITTER_KEY, int(PreviewType::ForwardVertical)).toInt());
 }
 
 PreviewWidget::~PreviewWidget()
 {
+  QSettings().setValue(PREVIEW_SPLITTER_KEY, static_cast<int>(_savedPreviewType));
   delete _image;
   delete _savedPreview;
 }
@@ -465,35 +467,35 @@ void PreviewWidget::paintSplittedPreview(QPainter & painter)
   case PreviewType::ForwardHorizontal:
     paintOriginalImage(painter);
     painter.save();
-    painter.setClipRect(0, y, position.width(), 1 + position.bottom() - y);
+    painter.setClipRect(position.left(), y, position.width(), 1 + position.bottom() - y);
     paintPreview(painter);
     painter.restore();
     break;
   case PreviewType::ForwardVertical:
     paintOriginalImage(painter);
     painter.save();
-    painter.setClipRect(x, 0, 1 + position.right() - x, position.height());
+    painter.setClipRect(x, position.top(), 1 + position.right() - x, position.height());
     paintPreview(painter);
     painter.restore();
     break;
   case PreviewType::BackwardHorizontal:
     paintPreview(painter);
     painter.save();
-    painter.setClipRect(0, y, position.width(), 1 + position.bottom() - y);
+    painter.setClipRect(position.left(), y, position.width(), 1 + position.bottom() - y);
     paintOriginalImage(painter);
     painter.restore();
     break;
   case PreviewType::BackwardVertical:
     paintPreview(painter);
     painter.save();
-    painter.setClipRect(x, 0, 1 + position.right() - x, position.height());
+    painter.setClipRect(x, position.top(), 1 + position.right() - x, position.height());
     paintOriginalImage(painter);
     painter.restore();
     break;
   case PreviewType::DuplicateTop:
     paintOriginalImage(painter);
     painter.save();
-    painter.setClipRect(0, y, position.width(), 1 + position.bottom() - y);
+    painter.setClipRect(position.left(), y, position.width(), 1 + position.bottom() - y);
     painter.translate(QPoint(0, y - position.top()));
     paintPreview(painter);
     painter.restore();
@@ -509,7 +511,7 @@ void PreviewWidget::paintSplittedPreview(QPainter & painter)
   case PreviewType::DuplicateLeft:
     paintOriginalImage(painter);
     painter.save();
-    painter.setClipRect(x, 0, 1 + position.right() - x, position.height());
+    painter.setClipRect(x, position.top(), 1 + position.right() - x, position.height());
     painter.translate(QPoint(x - position.left(), 0));
     paintPreview(painter);
     painter.restore();
@@ -517,7 +519,7 @@ void PreviewWidget::paintSplittedPreview(QPainter & painter)
   case PreviewType::DuplicateRight:
     paintOriginalImage(painter);
     painter.save();
-    painter.setClipRect(position.left(), 0, x, position.height());
+    painter.setClipRect(position.left(), position.top(), x, position.height());
     painter.translate(QPoint(x - position.right(), 0));
     paintPreview(painter);
     painter.restore();
@@ -530,7 +532,7 @@ void PreviewWidget::paintSplittedPreview(QPainter & painter)
     painter.translate(QPoint(0, (y - position.bottom()) / 2));
     paintOriginalImage(painter);
     painter.restore();
-    painter.setClipRect(0, y, position.width(), 1 + position.bottom() - y);
+    painter.setClipRect(position.left(), y, position.width(), 1 + position.bottom() - y);
     painter.translate(QPoint(0, (y - position.top()) / 2));
     paintPreview(painter);
     painter.restore();
@@ -543,7 +545,7 @@ void PreviewWidget::paintSplittedPreview(QPainter & painter)
     painter.translate(QPoint((x - position.right()) / 2, 0));
     paintOriginalImage(painter);
     painter.restore();
-    painter.setClipRect(x, 0, 1 + position.right() - x, position.height());
+    painter.setClipRect(x, position.top(), 1 + position.right() - x, position.height());
     painter.translate(QPoint((x - position.left()) / 2, 0));
     paintPreview(painter);
     painter.restore();
@@ -582,7 +584,7 @@ void PreviewWidget::paintEvent(QPaintEvent * e)
       paintSplittedPreview(painter);
     }
   }
-  if ((_previewType != PreviewType::Full) && _errorMessage.isEmpty()) {
+  if (_previewEnabled && (_previewType != PreviewType::Full) && _errorMessage.isEmpty()) {
     paintPreviewSplitter(painter);
   }
   e->accept();
@@ -1201,6 +1203,9 @@ void PreviewWidget::onPreviewToggled(bool on)
 void PreviewWidget::setPreviewType(PreviewType previewType)
 {
   _previewType = previewType;
+  if (previewType != PreviewType::Full) {
+    _savedPreviewType = previewType;
+  }
   setMouseTracking(previewType != PreviewType::Full);
   update();
 }
@@ -1232,6 +1237,16 @@ ZoomConstraint PreviewWidget::zoomConstraint() const
   return _zoomConstraint;
 }
 
+PreviewWidget::PreviewType PreviewWidget::previewType() const
+{
+  return _previewType;
+}
+
+PreviewWidget::PreviewType PreviewWidget::savedPreviewType() const
+{
+  return _savedPreviewType;
+}
+
 void PreviewWidget::invalidateSavedPreview()
 {
   _savedPreviewIsValid = false;
